import { describe, it, expect, beforeEach, afterEach } from 'vitest';
import { GraphService } from '../../services/GraphService.js';
import { ActionType } from '../../types/graphActions.js';
describe('GraphService', () => {
    let graphService;
    beforeEach(async () => {
        // Ensure clean test environment
        process.env.DEBUG_DATA_DIR = `.test-graph-service-${Date.now()}`;
        graphService = new GraphService();
        await graphService.initialize();
    });
    afterEach(async () => {
        // Clean up
        if (process.env.DEBUG_DATA_DIR) {
            const fs = await import('fs/promises');
            try {
                // Remove the entire test directory, not just the subdirectory
                await fs.rm(process.env.DEBUG_DATA_DIR, { recursive: true, force: true });
            }
            catch (error) {
                // Directory might not exist
            }
            delete process.env.DEBUG_DATA_DIR;
        }
    });
    describe('CREATE action', () => {
        it('should create a problem node', async () => {
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'App crashes on startup',
                metadata: {
                    tags: ['crash', 'startup'],
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.nodeId).toBeDefined();
            expect(response.message).toContain('Created problem node');
        });
        it('should create a hypothesis with parent problem', async () => {
            // First create a problem
            const problemResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak detected',
            });
            const problemResponse = JSON.parse(problemResult.content[0].text);
            expect(problemResponse.success).toBe(true);
            expect(problemResponse.nodeId).toBeDefined();
            // Then create a hypothesis
            const hypothesisResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Event listeners not being cleaned up',
                parentId: problemResponse.nodeId,
                metadata: {
                    confidence: 75,
                },
            });
            const hypothesisResponse = JSON.parse(hypothesisResult.content[0].text);
            expect(hypothesisResponse.success).toBe(true);
            expect(hypothesisResponse.nodeId).toBeDefined();
            expect(hypothesisResponse.edgeId).toBeDefined(); // Auto-created edge
        });
        it('should fail when parent node does not exist', async () => {
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Some hypothesis',
                parentId: 'non-existent-id',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(false);
            expect(response.message).toContain('Parent node non-existent-id not found');
        });
        it('should add root problem to roots array', async () => {
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Root problem',
            });
            const response = JSON.parse(result.content[0].text);
            const graph = graphService.getGraph();
            expect(graph.roots).toContain(response.nodeId);
        });
        it('should find similar problems when creating a new problem', async () => {
            // Create some existing problems first
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "x" of undefined',
                metadata: { status: 'solved' },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "y" of null',
                metadata: { status: 'solved' },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'ReferenceError: variable is not defined',
                metadata: { status: 'open' },
            });
            // Create a new problem similar to existing ones
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "z" of undefined',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.similarProblems).toBeDefined();
            expect(response.similarProblems.length).toBeGreaterThan(0);
            // Should find the TypeError problems with high similarity
            const typeErrors = response.similarProblems.filter((p) => p.content.includes('TypeError'));
            expect(typeErrors.length).toBeGreaterThanOrEqual(2);
            // Should have high similarity scores for TypeErrors
            expect(typeErrors[0].similarity).toBeGreaterThanOrEqual(0.6);
        });
        it('should not return similar problems for non-problem nodes', async () => {
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Some hypothesis content',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.similarProblems).toBeUndefined();
        });
        it('should include solutions with similar problems', async () => {
            // Create a problem with a solution
            const problemResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak in event listeners',
                metadata: { status: 'solved' },
            });
            const problemId = JSON.parse(problemResult.content[0].text).nodeId;
            const solutionResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Remove event listeners in cleanup function',
                metadata: { verified: true },
            });
            const solutionId = JSON.parse(solutionResult.content[0].text).nodeId;
            await graphService.connect({
                action: ActionType.CONNECT,
                from: solutionId,
                to: problemId,
                type: 'solves',
            });
            // Create a similar problem (more similar content for better matching)
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak detected in event listeners cleanup',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            // Check if similarProblems exists and has content
            if (response.similarProblems && response.similarProblems.length > 0) {
                // Should include the solution
                const problemWithSolution = response.similarProblems.find((p) => p.content.includes('Memory leak in event listeners'));
                if (problemWithSolution) {
                    expect(problemWithSolution.solutions).toBeDefined();
                    expect(problemWithSolution.solutions).toBeInstanceOf(Array);
                    // If solutions exist, check their content
                    if (problemWithSolution.solutions.length > 0) {
                        expect(problemWithSolution.solutions[0].content).toContain('Remove event listeners');
                    }
                }
            }
            else {
                // If no similar problems found, that's also acceptable for this test
                expect(response.similarProblems).toBeDefined();
            }
        });
        it('should prioritize solved problems in similar results', async () => {
            // Create multiple similar problems with different statuses
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'API timeout error on /users endpoint',
                metadata: { status: 'abandoned' },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'API timeout error on /products endpoint',
                metadata: { status: 'solved' },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'API timeout error on /orders endpoint',
                metadata: { status: 'open' },
            });
            // Create a new similar problem
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'API timeout error on /customers endpoint',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            // Check if similar problems were found
            if (response.similarProblems && response.similarProblems.length > 0) {
                // Find if there's a solved problem in the results
                const solvedProblem = response.similarProblems.find((p) => p.status === 'solved');
                if (solvedProblem) {
                    // Check if it's prioritized (should be in the first few results)
                    const solvedIndex = response.similarProblems.indexOf(solvedProblem);
                    expect(solvedIndex).toBeLessThanOrEqual(1); // Should be first or second
                }
            }
        });
        it('should calculate similarity correctly for error patterns', async () => {
            // Create problems with different error types
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "foo" of undefined',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'ReferenceError: foo is not defined',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot access property "bar" of null',
            });
            // Create a new TypeError
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "baz" of undefined',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            if (response.similarProblems && response.similarProblems.length > 0) {
                // TypeErrors should have higher similarity than ReferenceError
                const typeErrorProblems = response.similarProblems.filter((p) => p.content.includes('TypeError'));
                const refErrorProblems = response.similarProblems.filter((p) => p.content.includes('ReferenceError'));
                if (typeErrorProblems.length > 0 && refErrorProblems.length > 0) {
                    // Compare the highest similarity TypeError with the highest similarity ReferenceError
                    const maxTypeErrorSimilarity = Math.max(...typeErrorProblems.map((p) => p.similarity));
                    const maxRefErrorSimilarity = Math.max(...refErrorProblems.map((p) => p.similarity));
                    expect(maxTypeErrorSimilarity).toBeGreaterThan(maxRefErrorSimilarity);
                }
            }
        });
        it('should use error type index for performance optimization', async () => {
            // Create many problems with different error types
            for (let i = 0; i < 20; i++) {
                await graphService.create({
                    action: ActionType.CREATE,
                    nodeType: 'problem',
                    content: `TypeError: Cannot read property "${i}" of undefined`,
                });
            }
            for (let i = 0; i < 15; i++) {
                await graphService.create({
                    action: ActionType.CREATE,
                    nodeType: 'problem',
                    content: `ReferenceError: variable${i} is not defined`,
                });
            }
            for (let i = 0; i < 10; i++) {
                await graphService.create({
                    action: ActionType.CREATE,
                    nodeType: 'problem',
                    content: `Generic error ${i} without specific type`,
                });
            }
            // Create a new TypeError - should only search among TypeErrors
            const startTime = Date.now();
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property "test" of undefined',
            });
            const searchTime = Date.now() - startTime;
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.similarProblems).toBeDefined();
            // Should find similar TypeErrors
            const allTypeErrors = response.similarProblems.every((p) => p.content.includes('TypeError'));
            expect(allTypeErrors).toBe(true);
            // Performance should be reasonable even with many nodes
            expect(searchTime).toBeLessThan(100); // Should be fast with index
        });
        it('should handle problems without specific error types', async () => {
            // Create problems without error types
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Application crashes on startup',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak detected in production',
            });
            // Create another generic problem
            const result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Application hangs after 5 minutes',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            // Should still find similar problems even without error types
            if (response.similarProblems && response.similarProblems.length > 0) {
                expect(response.similarProblems[0].content).toBeDefined();
            }
        });
    });
    describe('CONNECT action', () => {
        it('should connect two nodes', async () => {
            // Create two nodes
            const node1Result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'observation',
                content: 'Error disappeared after fix',
            });
            const node1 = JSON.parse(node1Result.content[0].text);
            const node2Result = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'learning',
                content: 'Always clean up event listeners',
            });
            const node2 = JSON.parse(node2Result.content[0].text);
            // Connect them
            const connectResult = await graphService.connect({
                action: ActionType.CONNECT,
                from: node1.nodeId,
                to: node2.nodeId,
                type: 'learns',
                strength: 0.9,
            });
            const connectResponse = JSON.parse(connectResult.content[0].text);
            expect(connectResponse.success).toBe(true);
            expect(connectResponse.edgeId).toBeDefined();
            expect(connectResponse.message).toContain('Connected observation to learning');
        });
        it('should detect conflicting edges', async () => {
            // Create hypothesis and experiment
            const hypResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Theory A',
            });
            const hyp = JSON.parse(hypResult.content[0].text);
            const expResult = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'experiment',
                content: 'Test theory',
            });
            const exp = JSON.parse(expResult.content[0].text);
            // Create supporting edge
            await graphService.connect({
                action: ActionType.CONNECT,
                from: exp.nodeId,
                to: hyp.nodeId,
                type: 'supports',
            });
            // Create contradicting edge
            const contradictResult = await graphService.connect({
                action: ActionType.CONNECT,
                from: exp.nodeId,
                to: hyp.nodeId,
                type: 'contradicts',
            });
            const contradictResponse = JSON.parse(contradictResult.content[0].text);
            expect(contradictResponse.success).toBe(true);
            expect(contradictResponse.conflicts).toBeDefined();
            expect(contradictResponse.conflicts.conflictingEdges).toHaveLength(1);
        });
        it('should fail when nodes do not exist', async () => {
            const result = await graphService.connect({
                action: ActionType.CONNECT,
                from: 'non-existent-1',
                to: 'non-existent-2',
                type: 'supports',
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(false);
            expect(response.message).toContain('Node(s) not found');
        });
    });
    describe('QUERY action', () => {
        beforeEach(async () => {
            // Set up some test data
            const problem1 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeScript compilation error',
            });
            const p1 = JSON.parse(problem1.content[0].text);
            const solution1 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Fix type definitions',
            });
            const s1 = JSON.parse(solution1.content[0].text);
            await graphService.connect({
                action: ActionType.CONNECT,
                from: s1.nodeId,
                to: p1.nodeId,
                type: 'solves',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeScript type mismatch in React component',
            });
        });
        it('should find similar problems', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'similar-problems',
                parameters: {
                    pattern: 'typescript',
                    limit: 5,
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.problems).toBeDefined();
            // Check if we found problems containing 'typescript' (case-insensitive)
            if (response.results.problems.length > 0) {
                const hasTypescriptProblems = response.results.problems.some((p) => p.content.toLowerCase().includes('typescript'));
                expect(hasTypescriptProblems).toBe(true);
            }
        });
        it('should get node details', async () => {
            const problem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Test problem for details',
            });
            const p = JSON.parse(problem.content[0].text);
            // Create a hypothesis connected from the problem
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Test hypothesis',
                parentId: p.nodeId,
            });
            const h = JSON.parse(hypothesis.content[0].text);
            // Create a solution that solves the problem
            const solution = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Test solution',
            });
            const s = JSON.parse(solution.content[0].text);
            await graphService.connect({
                action: ActionType.CONNECT,
                from: s.nodeId,
                to: p.nodeId,
                type: 'solves',
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'node-details',
                parameters: {
                    nodeId: p.nodeId,
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results).toBeDefined();
            expect(response.results.node).toBeDefined();
            expect(response.results.node.content).toBe('Test problem for details');
            expect(response.results.incomingEdges).toBeDefined();
            expect(response.results.outgoingEdges).toBeDefined();
            expect(response.results.incomingEdges.length).toBeGreaterThan(0);
            expect(response.results.outgoingEdges.length).toBeGreaterThan(0);
        });
        it('should trace learning path', async () => {
            // Create a path: problem -> hypothesis -> experiment -> observation -> learning
            const problem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Path test problem',
            });
            const p = JSON.parse(problem.content[0].text);
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Path test hypothesis',
                parentId: p.nodeId,
            });
            const h = JSON.parse(hypothesis.content[0].text);
            const experiment = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'experiment',
                content: 'Path test experiment',
                parentId: h.nodeId,
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'learning-path',
                parameters: {
                    nodeId: p.nodeId,
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.path).toBeDefined();
            expect(response.results.path.length).toBeGreaterThanOrEqual(3);
        });
        it('should generate graph visualization', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'graph-visualization',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.format).toBe('mermaid');
            expect(response.results.content).toContain('graph TD');
            expect(response.results.nodeCount).toBeGreaterThan(0);
        });
        it('should handle unknown query type', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'unknown-query-type',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(false);
            expect(response.message).toContain('Unknown query type');
        });
        it('should find successful patterns', async () => {
            // Create a successful pattern: problem -> hypothesis -> solution
            const problem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Performance issue',
            });
            const problemId = JSON.parse(problem.content[0].text).nodeId;
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Memory leak',
                parentId: problemId,
            });
            const hypothesisId = JSON.parse(hypothesis.content[0].text).nodeId;
            const solution = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Fix memory leak',
            });
            const solutionId = JSON.parse(solution.content[0].text).nodeId;
            await graphService.connect({
                action: ActionType.CONNECT,
                from: solutionId,
                to: problemId,
                type: 'solves',
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'successful-patterns',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.patterns).toBeDefined();
        });
        it('should find failed hypotheses', async () => {
            // Create a failed hypothesis
            const problem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Bug in code',
            });
            const problemId = JSON.parse(problem.content[0].text).nodeId;
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Wrong theory',
                parentId: problemId,
                metadata: { confidence: 20 },
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'failed-hypotheses',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results).toBeDefined();
        });
        it('should find solution candidates', async () => {
            // Create a problem
            const problem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'TypeError: Cannot read property of undefined',
            });
            const problemId = JSON.parse(problem.content[0].text).nodeId;
            // Create solutions with highly similar content to meet the 0.3 similarity threshold
            const solution1 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Fix TypeError: Cannot read property by adding null checks',
                metadata: { verified: true },
            });
            const solution2 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Handle TypeError with optional chaining operator',
                metadata: { verified: false },
            });
            // Create an unrelated solution
            const solution3 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Fix memory leak in React component lifecycle',
                metadata: { verified: true },
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'solution-candidates',
                parameters: { nodeId: problemId },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.candidates).toBeDefined();
            expect(response.results.candidates).toBeInstanceOf(Array);
            expect(response.results.problemId).toBe(problemId);
            // Should find at least one solution candidate with TypeError
            expect(response.results.candidates.length).toBeGreaterThanOrEqual(1);
            // Verify the candidate structure
            if (response.results.candidates.length > 0) {
                const firstCandidate = response.results.candidates[0];
                expect(firstCandidate).toHaveProperty('solution');
                expect(firstCandidate).toHaveProperty('relevance');
                expect(firstCandidate).toHaveProperty('verified');
                expect(firstCandidate.relevance).toBeGreaterThan(0.3);
            }
        });
        it('should handle solution candidates with no node ID', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'solution-candidates',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.candidates).toEqual([]);
        });
        it('should handle solution candidates for non-problem node', async () => {
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Test hypothesis',
            });
            const hypothesisId = JSON.parse(hypothesis.content[0].text).nodeId;
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'solution-candidates',
                parameters: { nodeId: hypothesisId },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.candidates).toEqual([]);
        });
        it('should find related nodes', async () => {
            const node1 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Test problem',
            });
            const nodeId1 = JSON.parse(node1.content[0].text).nodeId;
            const node2 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Related hypothesis',
                parentId: nodeId1,
            });
            const nodeId2 = JSON.parse(node2.content[0].text).nodeId;
            const node3 = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'experiment',
                content: 'Test experiment',
            });
            const nodeId3 = JSON.parse(node3.content[0].text).nodeId;
            // Create edges in both directions
            await graphService.connect({
                action: ActionType.CONNECT,
                from: nodeId3,
                to: nodeId2,
                type: 'tests',
            });
            // Query for related nodes from the middle node (hypothesis)
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'related-nodes',
                parameters: { nodeId: nodeId2 },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results).toBeDefined();
            expect(response.results).toBeInstanceOf(Array);
            expect(response.results.length).toBeGreaterThanOrEqual(2); // Should find both problem and experiment
        });
        it('should handle pattern-match query', async () => {
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak in production',
                metadata: { tags: ['memory', 'production'] },
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'pattern-match',
                parameters: {
                    pattern: 'memory',
                    nodeTypes: ['problem'],
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results).toBeDefined();
            expect(response.results.matches.length).toBeGreaterThan(0);
        });
        it('should match patterns in tags', async () => {
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Some other issue',
                metadata: { tags: ['performance', 'critical'] },
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'pattern-match',
                parameters: {
                    pattern: 'critical',
                },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.matches.length).toBeGreaterThan(0);
            expect(response.results.matches[0].matchType).toBe('tag');
        });
        it('should handle pattern-match with no pattern', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'pattern-match',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.matches).toEqual([]);
        });
    });
    describe('Automatic edge creation', () => {
        it('should create correct edge types based on node types', async () => {
            // Problem -> Problem (decomposes)
            const rootProblem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Main problem',
            });
            const rp = JSON.parse(rootProblem.content[0].text);
            const subProblem = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Sub problem',
                parentId: rp.nodeId,
            });
            const sp = JSON.parse(subProblem.content[0].text);
            // Check that edge was created
            const graph = graphService.getGraph();
            const edge = Array.from(graph.edges.values()).find(e => e.from === rp.nodeId && e.to === sp.nodeId);
            expect(edge).toBeDefined();
            expect(edge?.type).toBe('decomposes');
            // Problem -> Hypothesis (hypothesizes)
            const hypothesis = await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Test hypothesis',
                parentId: sp.nodeId,
            });
            const h = JSON.parse(hypothesis.content[0].text);
            const hypEdge = Array.from(graph.edges.values()).find(e => e.from === sp.nodeId && e.to === h.nodeId);
            expect(hypEdge?.type).toBe('hypothesizes');
        });
    });
    describe('Graph persistence', () => {
        it('should save and load graph data', async () => {
            // Create some data
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Persistence test problem',
            });
            await graphService.saveGraph();
            // Create new instance and load
            const newGraphService = new GraphService();
            await newGraphService.initialize();
            const graph = newGraphService.getGraph();
            expect(graph.nodes.size).toBeGreaterThan(0);
            const hasPersistedProblem = Array.from(graph.nodes.values()).some(n => n.content === 'Persistence test problem');
            expect(hasPersistedProblem).toBe(true);
        });
    });
    describe('findFailedHypotheses', () => {
        it('should find hypotheses with low confidence', async () => {
            // Create hypotheses with different confidence levels
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Low confidence hypothesis',
                metadata: { confidence: 30 },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'High confidence hypothesis',
                metadata: { confidence: 80 },
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'hypothesis',
                content: 'Medium confidence hypothesis',
                metadata: { confidence: 45 },
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'failed-hypotheses',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.hypotheses).toBeDefined();
            expect(response.results.hypotheses.length).toBe(2); // Only confidence < 50
            expect(response.results.hypotheses[0].confidence).toBeLessThan(50);
        });
    });
    describe('findPatternMatches', () => {
        it('should find nodes matching content pattern', async () => {
            // Create nodes with different content
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Memory leak in production',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'observation',
                content: 'Memory usage keeps increasing',
            });
            await graphService.create({
                action: ActionType.CREATE,
                nodeType: 'solution',
                content: 'Fixed memory leak by clearing cache',
            });
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'pattern-match',
                parameters: { pattern: 'memory' },
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.matches).toBeDefined();
            expect(response.results.matches.length).toBe(3); // All contain "memory"
        });
        it('should handle empty pattern', async () => {
            const result = await graphService.query({
                action: ActionType.QUERY,
                type: 'pattern-match',
                parameters: {},
            });
            const response = JSON.parse(result.content[0].text);
            expect(response.success).toBe(true);
            expect(response.results.matches).toBeDefined();
            expect(response.results.matches.length).toBe(0);
        });
    });
    describe('buildErrorTypeIndex edge cases', () => {
        it('should handle nodes without error types', async () => {
            // Create a GraphService and add nodes without error types
            const service = new GraphService();
            await service.initialize();
            await service.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Generic problem without error type',
            });
            await service.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'Another generic issue',
            });
            // Force rebuild index
            const graph = service.getGraph();
            expect(graph.nodes.size).toBeGreaterThan(0);
            // @ts-ignore - access private method for testing
            service.buildErrorTypeIndex();
            // Verify that nodes without error types are categorized as 'other'
            // @ts-ignore - access private property for testing
            expect(service.errorTypeIndex.has('other')).toBe(true);
            // @ts-ignore
            expect(service.errorTypeIndex.get('other')?.size).toBeGreaterThan(0);
        });
        it('should create new error type entries when encountering new types', async () => {
            const service = new GraphService();
            await service.initialize();
            // First create a problem with a unique error type to ensure the index is empty for this type
            await service.create({
                action: ActionType.CREATE,
                nodeType: 'problem',
                content: 'URI Error: Malformed URI sequence',
            });
            // Force rebuild index
            // @ts-ignore - access private method for testing
            service.buildErrorTypeIndex();
            // Verify that new error type was added to index (the regex extracts "uri error")
            // @ts-ignore - access private property for testing
            expect(service.errorTypeIndex.has('uri error')).toBe(true);
        });
    });
});
//# sourceMappingURL=GraphService.test.js.map