import { NodeType, EdgeType } from './graph.js';
/**
 * グラフ操作用アクションタイプ
 * MCPツールが受け付ける3つの基本操作
 */
export declare enum ActionType {
    CREATE = "create",
    CONNECT = "connect",
    QUERY = "query"
}
/**
 * ベースアクションインターフェース
 * すべてのアクションが共通で持つactionフィールド
 */
interface BaseAction {
    action: ActionType;
}
/**
 * CREATEアクション: 新規ノードの作成
 * parentIdを指定すると適切なエッジを自動生成
 */
export interface CreateAction extends BaseAction {
    action: ActionType.CREATE;
    nodeType: NodeType;
    content: string;
    parentId?: string;
    metadata?: {
        confidence?: number;
        tags?: string[];
        [key: string]: any;
    };
}
/**
 * CONNECTアクション: ノード間の明示的な関係作成
 * 既存ノード間に意味的なエッジを追加
 */
export interface ConnectAction extends BaseAction {
    action: ActionType.CONNECT;
    from: string;
    to: string;
    type: EdgeType;
    strength?: number;
    metadata?: {
        reasoning?: string;
        evidence?: string;
        [key: string]: any;
    };
}
/**
 * クエリタイプ: グラフに対する検索・分析操作
 * 各タイプは異なる分析目的に特化
 */
export type QueryType = 'similar-problems' | 'successful-patterns' | 'failed-hypotheses' | 'learning-path' | 'solution-candidates' | 'graph-visualization' | 'node-details' | 'related-nodes' | 'pattern-match';
/**
 * QUERYアクション: グラフデータの検索・分析
 * パラメータで細かい検索条件を指定可能
 */
export interface QueryAction extends BaseAction {
    action: ActionType.QUERY;
    type: QueryType;
    parameters?: {
        nodeId?: string;
        pattern?: string;
        nodeTypes?: NodeType[];
        edgeTypes?: EdgeType[];
        confidence?: number;
        limit?: number;
        depth?: number;
        timeRange?: {
            start?: Date;
            end?: Date;
        };
        tags?: string[];
    };
}
/**
 * GraphActionユニオン型
 * MCPツールが受け付けるすべてのアクションの統合型
 */
export type GraphAction = CreateAction | ConnectAction | QueryAction;
/**
 * アクション実行結果のレスポンス型定義
 */
/**
 * CREATEアクションのレスポンス
 * 作成されたノード情報と提案を含む
 */
export interface CreateResponse {
    success: boolean;
    nodeId?: string;
    edgeId?: string;
    message?: string;
    suggestions?: {
        relatedProblems?: string[];
        possibleHypotheses?: string[];
        recommendedExperiments?: string[];
    };
    similarProblems?: Array<{
        nodeId: string;
        content: string;
        similarity: number;
        status?: 'open' | 'investigating' | 'solved' | 'abandoned';
        solutions: Array<{
            nodeId: string;
            content: string;
            verified: boolean;
        }>;
    }>;
}
/**
 * CONNECTアクションのレスポンス
 * 矛盾検出機能付き
 */
export interface ConnectResponse {
    success: boolean;
    edgeId?: string;
    message?: string;
    conflicts?: {
        conflictingEdges: string[];
        explanation: string;
    };
}
/**
 * QUERYアクションの汎用レスポンス
 * 実際の結果型はクエリタイプに依存
 */
export interface QueryResponse {
    success: boolean;
    results?: any;
    message?: string;
    queryTime?: number;
}
/**
 * クエリタイプ別の詳細レスポンス型
 * QueryResponse.resultsの実際の型定義
 */
/**
 * similar-problemsクエリの結果型
 * 類似度と解決策情報を含む問題リスト
 */
export interface SimilarProblemsResult {
    problems: Array<{
        nodeId: string;
        content: string;
        similarity: number;
        status?: 'open' | 'investigating' | 'solved' | 'abandoned';
        solutions?: Array<{
            nodeId: string;
            content: string;
            verified: boolean;
        }>;
    }>;
}
/**
 * successful-patternsクエリの結果型
 * 成功パターンの統計と具体例
 */
export interface SuccessfulPatternsResult {
    patterns: Array<{
        description: string;
        frequency: number;
        successRate: number;
        examplePaths: Array<{
            problem: string;
            hypothesis: string;
            experiment: string;
            solution: string;
        }>;
    }>;
}
/**
 * learning-pathクエリの結果型
 * ノード間の接続関係を含む経路情報
 */
export interface LearningPathResult {
    path: Array<{
        nodeId: string;
        type: NodeType;
        content: string;
        connections: Array<{
            type: EdgeType;
            to: string;
        }>;
    }>;
}
/**
 * graph-visualizationクエリの結果型
 * 様々な形式でのグラフ出力
 */
export interface GraphVisualizationResult {
    format: 'mermaid' | 'dot' | 'json';
    content: string;
    nodeCount: number;
    edgeCount: number;
}
/**
 * 親子関係から適切なエッジタイプを自動判定
 * CREATEアクションでparentId指定時に使用
 * @param parentType 親ノードのタイプ
 * @param childType 子ノードのタイプ
 * @returns 適切なエッジタイプまたはnull(マッピングなしの場合)
 */
export declare function getAutoEdgeType(parentType: NodeType, childType: NodeType): EdgeType | null;
export {};
//# sourceMappingURL=graphActions.d.ts.map